clear all;
close all;

% for experimental purpose
s = RandStream('mcg16807','Seed',121121); RandStream.setDefaultStream(s);

D    = 1000;         % dimensionality of the problem
NP   = 300;          % population size
runs = 1;            % repetitions
global initial_flag; % the global flag used in test suite

for func_num = 20%[20,18,16,15,13,11,10,8,6,5,3,2]
    
    if (contains(func_num, [2 5 10 15]))
        lb = -5;
        ub = 5;
    else
        if (contains(func_num, [3 6 11 16]))
            lb = -32;
            ub = 32;
        else
            lb = -100;
            ub = 100;
        end
    end
    
    for run = 1:runs
        close
        % should set the flag to 0 for each run, each function
        initial_flag = 0;
        pop = lb + (ub-lb) * rand(NP, D);
        
        % options for EDA, see eda.m for more details
        options.maxFE    = 6*10^(3); % 3*10^6; %1.2*10^5; %
        options.optmType = 'min';
        options.ub       = ub;
        options.lb       = lb;
        options.topSize  = 75;        
        options.covType  = 'full';
        options.k        = 3;
        options.rpmSize  = 1000;
        options.verbose  = true;
        
        t = cputime;
        % set the last argument (shift amount) to 'eps' to use the suite's
        % predefined shift values otherwise the function will be shifted by
        % 'shift amount'
        [x, val(run), stats, opts] = rp_eda(@benchmark_func, options, pop, func_num, 10);
        fprintf(1, 'Elapsed CPU time: %f\n', cputime-t);
        
        % demo output, you should save your results to some files
        fprintf(1, 'func_num = %d, run = %d\n', func_num, run);
        fprintf(1, 'min(val) = %f\n\n', min(val(run)));
        
        % saving often so we have a partial result even if the whole run doesn't finish
        save(['F' num2str(func_num)],'x','val','stats','opts');        
    end
end
